interface ArrayUtilInter {
    /**
     * 数组元素交换位置
     * @param {array} arr 数组
     * @param {number} index1 添加项目的位置
     * @param {number} index2 删除项目的位置
     * index1和index2分别是两个数组的索引值，即是两个要交换元素位置的索引值，如1，5就是数组中下标为1和5的两个元素交换位置
     */
    swapArray: (arr: Array<any>, index1: number, index2: number) => any[];
    /**
     * 上移
     * @param arr
     * @param index
     */
    zIndexUp: (arr: Array<any>, index: number) => void;
    /**
     * 下移
     * @param arr
     * @param index
     */
    zIndexDown: (arr: Array<any>, index: number) => void;
    /**
     * 置顶
     * @param arr
     * @param index
     */
    zIndexTop: (arr: Array<any>, index: number) => void;
    /**
     * 置底
     * @param arr
     * @param index
     */
    zIndexBottom: (arr: Array<any>, index: number) => void;
    /**
     * 获取差集
     * @param arr1
     * @param arr2
     * @returns
     */
    diff: (arr1: Array<any>, arr2: Array<any>) => any[];
}
declare var ArrayUtil: ArrayUtilInter;
interface Base64UtilInter {
    /**
     * 加密字符串为base64
     * @param input
     * @returns
     */
    encode: (input: any) => string;
    /**
     * 解密base64为字符串
     * @param input
     * @returns
     */
    decode: (input: any) => string;
}
declare var Base64Util: Base64UtilInter;
interface ByteUtilInter {
    /**
     * 将byte数组转为字符串
     * @param str
     * @returns
     */
    byteToString: (arr: any) => string;
    /**
     * 将字符串转为byte数组
     * @param str
     * @returns
     */
    stringToByte: (str: any) => any[];
    /**
     * 将Uint8Array转为字符串
     * @param fileData
     * @returns
     */
    uint8ArrayToString(fileData: any): string;
    /**
     * 将字符串转为Uint8Array
     * @param str
     * @returns
     */
    stringToUint8Array(str: any): Uint8Array;
}
declare var ByteUtil: ByteUtilInter;
/**
 * 全局事件管理
 */
interface CEventInter {
    /**
     * 注册
     * @param key 注册名称
     * @param callback 执行回调
     * @param target 附带信息
     * @returns 返回唯一注册id
     */
    on(key: any, callback: Function, ...target: any): string;
    /**
     * 调用
     * @param key 注册名称
     * @param data 回调函数传入参数
     */
    emit(key: any, data?: any): void;
    /**
     * 销毁
     * @mode 模式1:多id或者key名称传入-off('onid1')、off('onid1','getData'...)
     * @mode 模式2:on方法传入的key和方法-off('getData',fun)
     * @param uids-传入的是id或者key加方法清理的是key的单个注册，传入的是key清理key所有注册
     */
    off(...uids: any): void;
    /**
     * 清空
     */
    clear(): void;
}
declare var CEvent: CEventInter;
/**
 * 事件管理对象
 */
interface CEventBeanInter {
    /**
     * 事件id
     */
    /**
     * 获取事件id
     * @returns
     */
    /**
     * 注册
     * @param key 注册名称
     * @param callback 执行回调
     * @param target 附带信息
     * @returns 返回唯一注册id
     */
    on: (key: any, callback: any, ...target: any) => string;
    /**
     * 调用
     * @param key 注册名称
     * @param data 回调函数传入参数
     */
    emit: (key: any, data: any) => void;
    /**
     * 销毁
     * @mode 模式1:多id或者key名称传入-off('onid1')、off('onid1','getData'...)
     * @mode 模式2:on方法传入的key和方法-off('getData',fun)
     * @param uids-传入的是id或者key加方法清理的是key的单个注册，传入的是key清理key所有注册
     */
    off: (...uids: any) => void;
    clear: () => void;
}
interface CEventBeanObj { new ():CEventBeanInter}
declare var CEventBean: CEventBeanObj;
interface CMapInter<T = any> {
    obj: {
        [key: string]: T;
    };
    get(key: string | number): T;
    has(key: string | number): boolean;
    set(key: string | number, value: T): void;
    /**
     *  遍历，返回true时停止遍历
     * @param callback -回调value,key,index
     * @example
     * cmap.forEach((key,value,index)=>{
     *  console.log(value)
     *  return index === 0
     * })
     */
    forEach(callback: (key: string, value: T, index: number) => unknown): void;
    /**
     * 获取key列表
     * @returns
     */
    keys(): string[];
    /**
     * 获取值列表
     * @returns
     */
    values(): T[];
    /**
     * 移除key
     * @returns
     */
    remove(key: string): void;
    /**
     * 获取map大小
     * @returns
     */
    size(): number;
    /**
     * 清空map
     * @returns
     */
    clear(): void;
    /**
     * 转为json字符串
     * @returns
     */
    toJson(): string;
}
interface CMapObj { new <T = any>():CMapInter<T>}
declare var CMap: CMapObj;
/**
 * Cookie工具
 */
interface CookieInter {
    /**
     * 设置cookie
     * @param name
     * @param value
     * @param config
     */
    set(name: string, value: string, config?: {
        expire?: number;
        domain?: string;
    }): void;
    /**
     * 获取cookie
     * @param name
     * @returns
     */
    get(name: string): string | null;
    /**
     * 删除cookie
     * @param name
     * @param domain
     */
    remove(name: string, domain?: string): void;
}
declare var Cookie: CookieInter;
interface DomUtilInter {
    /**
     * 对节点添加监听事件和回调
     * @param dom
     * @param type
     * @param listener
     * @returns
     */
    listener: <K extends keyof GlobalEventHandlersEventMap>(dom: Node | Window, type: K, listener: (this: Node, ev: GlobalEventHandlersEventMap[K]) => any) => {
        remove: () => void;
    };
    /**
     * 获取管理点击或者触摸事件管理对象
     * @returns
     */
    useTouch: () => {
        move: (event: TouchEvent | Touch | Event | MouseEvent) => void;
        start: (event: TouchEvent | Touch | Event | MouseEvent) => void;
        reset: () => void;
        startX: number;
        startY: number;
        deltaX: number;
        deltaY: number;
        offsetX: number;
        offsetY: number;
        direction: "";
        isVertical: () => boolean;
        isHorizontal: () => boolean;
    };
    /**
     *
     * @param div 目标，需要是absolute
     * @param fun 移动完毕之后的回调
     */
    moveDiv(div: HTMLDivElement, onmouseup?: (data: {
        left: number;
        top: number;
    }) => any): void;
    /**
     * 获取滚动条宽度
     */
    getScrollbarWidth(): number;
}
declare var DomUtil: DomUtilInter;
/**
 * 简单加解密工具
 */
interface EnUtilInter {
    /**
     * 加密
     * @param str
     * @returns
     */
    encode: (str: string) => string;
    /**
     * 解密
     * @param str
     * @returns
     */
    decode: (str: string) => string;
}
declare var EnUtil: EnUtilInter;
/**
 * @type protype
 */
interface Array<T> {
    /**
     * 移除数组中的指定内容，用于不是对象素组
     * @param str 删除内容
     * @example
     * let arr = new Array<string>()
     * arr.push("1")
     * arr.push("2")
     * arr.remove("1")
     */
    remove(item: (val: T) => any): void;
    /**
     * 去重
     * @param predicate 比较说明
     * @returns
     * @example
     * const arr1 = arr.toSet()
     * const arr2 = arr.toSet((next,pre)=>next.id===pre.id)
     */
    toSet(predicate?: (next: T, pre: T) => any): any[];
    /**
     * 随机排序
     */
    randSort(): void;
    /**
     * 累加
     * @param field -对象字段名称
     * @example
     * const num = arr.sum()
     * const num = arr.sum('num')
     */
    sum(field?: string): any;
}
/**
 * @type protype
 */
interface Date {
    /**
     * 格式化时间
     * @param format 时间字符串格式
     * @example
     * new Date().Format('yyyy-MM-dd hh:mm:ss') //2021-12-17 11:30:00
     */
    Format(fmt?: any): any;
    /**
     * 获取时间是星期几，默认获取今天
     */
    getWeek(date?: Date): string;
}
/**
 * @type protype
 */
interface DOMRect {
    /**
     * x,y坐标是否处于矩形内
     * @param x
     * @param y
     */
    contains(x: number, y: number): boolean;
    /**
     * x坐标是否处于矩形内
     * @param x
     */
    containX(x: number): boolean;
    /**
     * y坐标是否处于矩形内
     * @param y
     */
    containY(y: number): boolean;
}
/**
 * @type protype
 */
interface Function {
    /**
     * 获取参数列表
     */
    args(): string[];
}
/**
 * @type protype
 */
interface Map<K, V> {
    /**
     * 将map输出为json
     */
    toJson(): string;
}
/**
 * iframe标签简单交互工具
 */
interface IFrameUtilInter {
    /**
     * 发送数据
     * @param data
     * @param childrenId -传入id时向iframe子节点传输数据，反之向父级传输数据
     */
    sendMessage: (data: any, childrenId?: string) => void;
    /**
     * 注册接收数据
     * @param fun
     */
    onMassage: (fun: (data: any) => void) => void;
    /**
     * 销毁接收数据
     * @param fun
     */
    offMassage: (fun: (data: any) => void) => void;
}
declare var IFrameUtil: IFrameUtilInter;
interface JSONUtilInter {
    /**
     * 复制json对象
     */
    cp<T>(obj: T): T;
    /**
     * 深度拷贝-对象、数组、时间、正则
     * @param objT
     * @param hash
     * @returns
     */
    deepClone<T>(objT: T, hash?: WeakMap<object, any>): T;
}
declare var JSONUtil: JSONUtilInter;
interface KeyUtilInter {
    /**
     * 监听组合键
     * @param keys 组合键数组
     * @param callback 触发回调函数
     * @example
     * //Ctrl+Q
     * groupKey(['Ctrl','Q'],()=>{
     *  console.log('成功')
     * })
     * @returns
     */
    groupKey: (keys: string[], callback: Function) => () => void;
}
declare var KeyUtil: KeyUtilInter;
interface MathUtilInter {
    /**
     * 将所有值转换为float
     */
    getNumber(val: any, fixed?: number): number;
    /**
     * 获取两个整数之间的随机数
     */
    getRandomInt(min: number, max: number): number;
}
declare var MathUtil: MathUtilInter;
interface MsgUtilInter {
    Message: any;
    /**
     * 成功通知
     */
    sr(content: string): void;
    /**
     * 失败通知
     */
    er(content: string): void;
}
declare var MsgUtil: MsgUtilInter;
interface ScriptUtilInter {
    /**
     * 加载js文件到页面
     * @param url
     * @returns
     */
    scriptLoad(url: any): Promise<boolean>;
}
declare var ScriptUtil: ScriptUtilInter;
interface StrUtilInter {
    /**
     * 获取uuid
     * @returns
     */
    uuid(): string;
    /**
     * 获取唯一递增id
     */
    getId: () => string;
    /**
     * 判断内容是否为空,null,undefined,NaN，空数组，空字符串返回true
     */
    isNull(str: any): boolean;
    /**
     * 验证邮箱
     */
    isEmail(obj: any): boolean;
    /**
     * 验证数字
     */
    isNumber(obj: any): boolean;
    /**
     * 获取url地址中的参数对象信息
     * @param url 如/list?pageSize=1&pageNum=2
     * @param return 如{pageSize:1,pageNum:2}不符合返回{}
     */
    getParam(url: any): any;
}
declare var StrUtil: StrUtilInter;
/**
 * 计时器管理
 */
interface TimerInter {
    map: CMapInter;
    delay(millisecond?: number): Promise<void>;
    /**
     * 执行列表
     */
    run: (...run: {
        run: () => void;
        time?: number | undefined;
    }[]) => void;
    /**
     * 异步执行列表,执行完所有任务会调用回调
     */
    runAsync: (run: {
        run: () => void;
        time?: number | undefined;
    }[], callback?: any) => Promise<void>;
    /**
        循环执行
     */
    on: (callback: Function, time?: number, runOne?: boolean) => string;
    /**
     * 执行一次
     */
    once: (callback: Function, time?: number) => string;
    /**
     * 异步执行一次
     */
    onceAsync: (callback: Function, time?: number) => Promise<void>;
    /**
        注销循环
     */
    un(...name: any): void;
    /**
        注销一次
     */
    unOnce(...name: any): void;
    /**
        注销所有循环
     */
    unAll(): void;
}
declare var Timer: TimerInter;

type $Constructor = {
    new(): void
    /**
     * @param selector -选择器字符串(支持如 '#main'、'.text'、'#main .class')
     * @param one -是否获取第一个元素，默认是true
     * @param context -查找源，默认是document
     */
    (selector: string, one?: boolean, context?: Document): Element | HTMLCollectionOf<Element> | (Element | HTMLCollectionOf<Element>)[];
}

declare var $: $Constructor;/**
 * 传入参数和数据
 */
type THttpReq = <T = any>(param: string | { 
    /**
     * 请求地址
     */
    url: string; 
    /**
     * 请求类型
     */
    method?: "POST"|"GET"|"PUT"|"DELETE"|"OPTIONS"|"HEAD"|"PATCH"; 
    /**
     * 消息类型-默认text
     */
    responseType?: XMLHttpRequestResponseType
    /**
     * 超时时间-默认30000
     */
    timeout?: number;
    /**
     * 是否跨域传入cookie-默认false
     */
    
    withCredentials?: boolean;
 }, data?: any) => Promise<T>

interface THttpConfig {
    /**
     * 根地址，当传入的url以http开头，此地址无效
     */
    base: string;
    /**
     * 超时时间-默认30000
     */
    timeout: number;
    /**
     * 消息类型-默认text
     */
    responseType: XMLHttpRequestResponseType;
    /**
     * 是否跨域传入cookie-默认false
     */
    
    withCredentials: boolean;
    /**
     * 标头
     * @example
     * //默认
     * {
            Accept: '*',
            'Content-Type': 'application/json'
        }
     */
    headers: {
        [key: string]: any;
    };
    /**
     * 请求前处理
     * @param config 
     * @returns 
     */
    before: (config: {
        /**
         * 标头
         */
        headers: {
            [key: string]: any;
        };
        /**
         * 传入数据
         */
        data: any;
        /**
         * 传入参数
         */
        param: any;
        /**
         * 中断当前请求
         */
        stop: () => void;
    }) => void | Promise<any>;
    /**
     * 请求后处理
     * @param xhr 
     * @returns 
     */
    after: (xhr:{
        /**
         * 获取数据
         */
        data: any;
        /**
         * 消息类型
         */
        responseType: XMLHttpRequestResponseType;
        /**
         * 状态码
         */
        status: number;
    }) => void | Promise<any>;
    /**
     * 异常处理
     * @param e 
     * @returns 
     */
    error: (status:number) => void;
}

/**
 * 全局请求对象
 */
interface HttpInter {
    /**
     * 请求管理器
     */
    xhr: {
        [key: string]: XMLHttpRequest;
    };
    /**
     * get请求
     */
    get: THttpReq;
    /**
     * post请求
     */
    post: THttpReq;
    /**
     * 中断所有请求
     */
    stop: () => void;

    /**
     * 配置信息
     */
    config: THttpConfig;

    /**
     * 设置配置信息
     */
    setConfig:(config?:Partial<THttpConfig>)=>void
}

/**
 * 全局请求对象
 */
declare var http:HttpInter;

/**
 * 获取一个http请求对象
 */
declare var httpBean:(config?:Partial<THttpConfig>)=>HttpInter;interface Window {
    [key:string]:any
}